searchData={"items":[{"type":"behaviour","title":"Confeature","doc":"Defines a macro for implementing your own feature settings store on top of\nEcto and the cache store of your choice.\n\nYou can simply declare your module like this:\n\n    defmodule MyApp.Confeature do\n      use Confeature,\n          ecto_repo: MyApp.Repo,\n          cache: MyApp.Cache.Feature\n    end\n\nProviding an ecto_repo is mandatory, but by default, Confeature will use a\nplaceholder cache module. Please refer to the `Confeature.Cache` module doc for detailed instructions\non how to implement your cache.\n\nThe functions docs below assume that you have the following feature modules\ndeclared, as references:\n\n    defmodule MyApp.Features.UsageAlert do\n      defstruct [:threshold]\n\n      @type t :: %__MODULE__{\n        threshold: float()\n      }\n    end\n\n    defmodule MyApp.Features.HiddenPixel do\n      defstruct [:enabled, :x, :y]\n\n      @type t :: %__MODULE__{\n        enabled: boolean(),\n        x: integer(),\n        y: integer()\n      }\n    end","ref":"Confeature.html"},{"type":"callback","title":"Confeature.delete!/1","doc":"Deletes the feature row from your database and invalidates the cache.\n\nYou may want to call this function once you're completely done with a\nfeature (eg. in a post-release Ecto migration).","ref":"Confeature.html#c:delete!/1"},{"type":"callback","title":"Confeature.disable/1","doc":"Disables a feature. This is a helper function, and it requires you to\ndeclare the enabled boolean field on the feature struct.\n\n    MyApp.Confeature.disable(MyApp.Features.UsageAlert)\n    => *will throw an error*\n\n    MyApp.Confeature.disable(MyApp.Features.HiddenPixel)\n    => {:ok, *your_updated_struct*}","ref":"Confeature.html#c:disable/1"},{"type":"callback","title":"Confeature.enable/1","doc":"Enables a feature. This is a helper function, and it requires you to\ndeclare the enabled boolean field on the feature struct.\n\n    MyApp.Confeature.enable(MyApp.Features.UsageAlert)\n    => *will throw an error*\n\n    MyApp.Confeature.enable(MyApp.Features.HiddenPixel)\n    => {:ok, *your_updated_struct*}","ref":"Confeature.html#c:enable/1"},{"type":"callback","title":"Confeature.enabled?/1","doc":"Returns true if your feature is enabled. This is a helper function, and\nit requires you to declare the enabled boolean field on the feature\nstruct.\n\n    MyApp.Confeature.enabled?(MyApp.Features.UsageAlert)\n    => *will throw an error*\n\n    MyApp.Confeature.enabled?(MyApp.Features.HiddenPixel)\n    => false","ref":"Confeature.html#c:enabled?/1"},{"type":"callback","title":"Confeature.get/1","doc":"Returns a feature struct based on the module name.\n\n    MyApp.Confeature.get(MyApp.Features.UsageAlert)\n    # If you never set it:\n    => nil\n\n    # After setting it:\n    => %MyApp.Features.UsageAlert{threshold: 4.5}","ref":"Confeature.html#c:get/1"},{"type":"callback","title":"Confeature.set/1","doc":"Writes new settings using a feature struct. It'll write to the database\nand invalidate cache right after.\n\nThis function allows incremental updates; parameters you don't provide\nwon't get erased.\n\n    MyApp.Confeature.set(%MyApp.Features.HiddenPixel{x: 43, y: 219})\n    => {:ok, *your_updated_struct*}","ref":"Confeature.html#c:set/1"},{"type":"behaviour","title":"Confeature.Cache","doc":"A behaviour module for implementing a key-value store cache. Rather than\nimplementing its own, `Confeature` lets you pick the implementation and\nleaves the details at your own discretion. Callbacks will be executed to\navoid sending queries to the database on each call of your `Confeature`\nstore.\n\nHere's an example, that actually doesn't cache anything:\n\n    defmodule MyApp.Cache.Feature do\n      @behaviour Confeature.Cache\n\n      @impl true\n      def get(_name), do: nil\n\n      @impl true\n      def set(_name, struct) do\n        {:ok, struct}\n      end\n\n      @impl true\n      def delete(name), do: {:ok, name}\n    end\n\nAnd here's an example of a cache implemented using ConCache (on top of ETS):\n\n    defmodule MyApp.Cache.Feature do\n      @behaviour Confeature.Cache\n\n      def child_spec() do\n        Supervisor.child_spec(\n          {\n            ConCache,\n            name: __MODULE__,\n            ttl_check_interval: :timer.minutes(1),\n            global_ttl: :timer.minutes(60),\n            touch_on_read: false\n          },\n          id: __MODULE__\n        )\n      end\n\n      @impl true\n      def get(name), do: ConCache.get(__MODULE__, name)\n\n      @impl true\n      def set(name, data) do\n        :ok = ConCache.put(__MODULE__, name, data)\n        {:ok, data}\n      end\n\n      @impl true\n      def delete(name), do: ConCache.delete(__MODULE__, name)\n    end\n\nIf necesary, you can of course use a PubSub such as the one provided by\nPhoenix, in case you'd deploy an app with multiple Erlang nodes.\n\nThere's also an example with Redis (via Redix) in the test suite of the\nConfeature Git repository.","ref":"Confeature.Cache.html"},{"type":"callback","title":"Confeature.Cache.delete/1","doc":"","ref":"Confeature.Cache.html#c:delete/1"},{"type":"callback","title":"Confeature.Cache.get/1","doc":"","ref":"Confeature.Cache.html#c:get/1"},{"type":"callback","title":"Confeature.Cache.set/2","doc":"","ref":"Confeature.Cache.html#c:set/2"},{"type":"module","title":"Confeature.Cache.Default","doc":"Default (and void) caching strategy.","ref":"Confeature.Cache.Default.html"},{"type":"module","title":"Confeature.Migration","doc":"Defines an Ecto migration to create the necesary table to store your feature\nsettings.\n\nLook at the [README](README.md) for detailed setup instructions.","ref":"Confeature.Migration.html"},{"type":"module","title":"Ecto Prefixes and SQL Schemas - Confeature.Migration","doc":"Confeature has been tested with Ecto multi-tenancy and should work in all\nscenarios, since it just sends basic Ecto queries that should be decorated\nby your Ecto repository default configuration.\n\nIf migrations fail, your Repo probably lacks the following callback:\n\n    defmodule MyApp.MyRepo do\n      use Ecto.Repo, ...\n\n      def default_options(_operation) do\n        [prefix: \"your_sql_prefix\"]\n      end\n    end","ref":"Confeature.Migration.html#module-ecto-prefixes-and-sql-schemas"},{"type":"function","title":"Confeature.Migration.down/1","doc":"Drops the Confeature table. You can set the following options:\n\n  * `table_name`: lets you specify the table name. Default is `features`.","ref":"Confeature.Migration.html#down/1"},{"type":"function","title":"Confeature.Migration.up/1","doc":"Creates Confeature table. You can set the following options:\n\n  * `table_name`: lets you specify the table name. Default is `features`.","ref":"Confeature.Migration.html#up/1"},{"type":"module","title":"Confeature.SQL","doc":"Internal module to perform SQL operations.","ref":"Confeature.SQL.html"},{"type":"function","title":"Confeature.SQL.delete/2","doc":"","ref":"Confeature.SQL.html#delete/2"},{"type":"function","title":"Confeature.SQL.disable/2","doc":"","ref":"Confeature.SQL.html#disable/2"},{"type":"function","title":"Confeature.SQL.enable/2","doc":"","ref":"Confeature.SQL.html#enable/2"},{"type":"function","title":"Confeature.SQL.get/2","doc":"","ref":"Confeature.SQL.html#get/2"},{"type":"function","title":"Confeature.SQL.upsert/2","doc":"","ref":"Confeature.SQL.html#upsert/2"},{"type":"module","title":"Confeature.Schema","doc":"Defines an internal Ecto schema and changeset used to store Features.","ref":"Confeature.Schema.html"},{"type":"function","title":"Confeature.Schema.changeset/2","doc":"","ref":"Confeature.Schema.html#changeset/2"},{"type":"type","title":"Confeature.Schema.t/0","doc":"","ref":"Confeature.Schema.html#t:t/0"},{"type":"module","title":"Confeature.Type","doc":"Defines an internal custom Ecto.Type that lets Ecto cast a Feature module\ninto a valid SQL fragment, and vice-versa.","ref":"Confeature.Type.html"},{"type":"function","title":"Confeature.Type.cast/1","doc":"","ref":"Confeature.Type.html#cast/1"},{"type":"function","title":"Confeature.Type.dump/1","doc":"","ref":"Confeature.Type.html#dump/1"},{"type":"function","title":"Confeature.Type.embed_as/1","doc":"","ref":"Confeature.Type.html#embed_as/1"},{"type":"function","title":"Confeature.Type.equal?/2","doc":"","ref":"Confeature.Type.html#equal?/2"},{"type":"function","title":"Confeature.Type.load/1","doc":"","ref":"Confeature.Type.html#load/1"},{"type":"function","title":"Confeature.Type.type/0","doc":"","ref":"Confeature.Type.html#type/0"},{"type":"extras","title":"Confeature","doc":"# Confeature\n\n`Confeature` is a generic and simple Elixir library to handle your application-wide feature settings, storing them with Ecto and providing a cache interface.\n\nPlease note that this is a very early development, and that `Confeature` hasn't been tested a lot in production yet.\n\nAlso, if you're looking for feature flags that you may apply on groups or actors, you should definitely use `:fun_with_flags` instead.\n\nThe scope of `Confeature` is primarily set on:\n* Allowing you to change your application settings dynamically,\n* Setting any value at runtime,\n* Type-checking updates with structs and specs.","ref":"readme.html"},{"type":"extras","title":"Documentation - Confeature","doc":"Documentation is available on [https://hexdocs.pm/confeature](https://hexdocs.pm/confeature).","ref":"readme.html#documentation"},{"type":"extras","title":"Installation - Confeature","doc":"Add `confeature` to your dependencies:\n```elixir\ndef deps do\n  [\n    {:confeature, \"~> 0.1.1\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Confeature","doc":"Start by creating Confeature table:\n```sh\nmix ecto.gen.migration create_confeature_table\n```\n\nAnd use Confeature provided migration:\n```elixir\ndefmodule YourEctoMigration do\n  def up do\n    Confeature.Migration.up()\n    # or optionally:\n    Confeature.Migration.up(table_name: \"my_features\")\n  end\n\n  def down do\n    Confeature.Migration.down()\n    # or optionally:\n    Confeature.Migration.down(table_name: \"my_features\")\n  end\nend\n```\n\nDeclare your Confeature interface\n```elixir\ndefmodule MyApp.Confeature do\n  use Confeature,\n    ecto_repo: MyApp.Repo,\n    table_name: \"my_features\", # Optional\n    cache: MyApp.Cache.Feature # Optional\nend\n```\nYou can check the documentation for implementing a cache store that'll avoid querying your database on each call.\n\nThen, declare a feature, like this:\n```elixir\ndefmodule MyApp.Features.Throttling do\n  defstruct [:identifier, :threshold]\n\n  @type t :: %__MODULE__{\n    identifier: string(),\n    threshold: integer()\n  }\nend\n```\n\nLet's say that you'd want to initialize it in an Ecto migration:\n```sh\nmix ecto.gen.migration init_throttling_settings\n```\n\n```elixir\ndefmodule YourMigration do\n  def up do\n    MyApp.Confeature.set(%MyApp.Features.Throttling{\n      identifier: \"token\",\n      threshold: 500 # 500 requests\n    })\n  end\n\n  def down do\n    MyApp.Confeature.delete!(MyApp.Features.Throttling)\n  end\nend\n```\n\nYou can then reference it in your code:\n```elixir\n# Retrieve settings\niex> MyApp.Confeature.get(MyApp.Features.Throttling)\n%MyApp.Features.Throttling{identifier: \"token\", threshold: 500}\n\niex> MyApp.Confeature.set(%MyApp.Features.Throttlin{identifier: \"token\", threshold: 1000})\n%MyApp.Features.Throttling{identifier: \"token\", threshold: 1000}\n```\n\nConfeature upserts one row per feature in your RDBMS, using a json field to store attributes.\n\nYou can also declare a `:enabled` attribute, so your feature can be enabled and disabled at runtime:\n```elixir\ndefmodule MyApp.Features.Throttling do\n  defstruct [:enabled, :identifier, :threshold]\n\n  @type t :: %__MODULE__{\n    enabled: boolean(),\n    identifier: string(),\n    threshold: integer()\n  }\nend\n\nMyApp.Confeature.enabled?(MyApp.Features.Throttling) \nMyApp.Confeature.enable(MyApp.Features.Throttling) \nMyApp.Confeature.disable(MyApp.Features.Throttling) \n```","ref":"readme.html#usage"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,49]}}